"""Tests for the lander.config module."""

import pytest

from lsstprojectmeta.tex.lsstdoc import LsstLatexDoc

from lander.config import Configuration
from lander.exceptions import DocuShareError


@pytest.mark.skip(reason='Fails because ls.st+DocuShare is broken (2017-09-07')
def test_get_docushare_url():
    url = Configuration._get_docushare_url('LDM-151')
    assert url == 'https://ls.st/ldm-151*'


def test_get_docushare_url_bad_handle():
    with pytest.raises(DocuShareError):
        Configuration._get_docushare_url('FOOBAR-1')


@pytest.mark.skip(reason='Fails because ls.st+DocuShare is broken (2017-09-07')
@pytest.mark.parametrize(
    'doc_handle, expected_url',
    [('LDM-151', 'https://ls.st/ldm-151*'),
     ('FOOBAR-1', None)])
def test_docushare_url_config(doc_handle, expected_url):
    """Integrated testing of doc_handle computation."""
    config = Configuration(doc_handle=doc_handle, _validate_pdf=False)
    if expected_url is None:
        assert config['docushare_url'] is None
    else:
        assert config['docushare_url'] == expected_url


@pytest.mark.parametrize(
    'git_branch, docclass, expected',
    [('u/jonathansick/test', r'\documentclass[DM]{lsstdoc}', False),
     ('u/jonathansick/test', r'\documentclass[DM,lsstdraft]{lsstdoc}', True),
     ('master', r'\documentclass[DM]{lsstdoc}', False),
     # master is always considered released
     ('master', r'\documentclass[DM,lsstdraft]{lsstdoc}', False),
     ('docushare-v1', r'\documentclass[DM]{lsstdoc}', False),
     ('docushare-v1', r'\documentclass[DM,lsstdraft]{lsstdoc}', True)])
def test_determine_draft_status(git_branch, docclass, expected):
    lsstdoc = LsstLatexDoc(docclass)
    is_draft = Configuration._determine_draft_status(git_branch,
                                                     lsstdoc=lsstdoc)
    assert is_draft == expected


@pytest.mark.parametrize(
    'git_branch, expected',
    [('u/jonathansick/test', True),
     ('master', False),
     ('docushare-v1', True),
     ('docushare-v10', True),
     ('docushare-v10-test', True)])
def test_draft_status(git_branch, expected):
    """Integrated testing of is_draft_branch computation. LsstLatexDoc metadata
    is not available here, only branch information is used.
    """
    config = Configuration(git_branch=git_branch, _validate_pdf=False)
    assert config['is_draft_branch'] is expected



"""Command line interface for ``lander`` executable."""
from argparse import ArgumentParser
import logging
import os
import sys

import pkg_resources
import structlog

from .config import Configuration
from .lander import Lander


def parse_args():
    """Create an `argparse.ArgumentParser` and parse command line arguments.
    """
    parser = ArgumentParser(
        description="Create a landing website for for PDF documentation.",
        epilog="lander is a product of LSST Data Management. Code is "
               "available at https://github.com/lsst-sqre/lander.")

    parser.add_argument(
        '--version',
        dest='show_version',
        default=False,
        action='store_true',
        help="Show version information and exit.")

    parser.add_argument(
        '--verbose',
        default=False,
        action='store_true',
        help='Show debugging information.')

    parser.add_argument(
        '--build-dir',
        dest='build_dir',
        default=os.path.abspath('_build'),
        help='Build directory (scratch space).')

    parser.add_argument(
        '--pdf',
        dest='pdf_path',
        help='Filepath of PDF document.'
    )

    parser.add_argument(
        '--lsstdoc',
        dest='lsstdoc_tex_path',
        help='File path of a lsstdoc LaTeX file (for metadata).'
    )

    parser.add_argument(
        '--env',
        dest='environment',
        choices=['travis'],
        default=None,
        help='Environment for auto-configuration'
    )

    parser.add_argument(
        '--title',
        dest='title',
        help='Document title (metadata override)'
    )

    parser.add_argument(
        '--authors',
        dest='authors_json',
        help='Author list, as a JSON array of strings (metadata override).'
    )

    parser.add_argument(
        '--abstract',
        dest='abstract',
        help='Document abstract (metadata override).'
    )

    parser.add_argument(
        '--handle',
        dest='doc_handle',
        help='Document handle, such as LDM-000 (metadata override).'
    )

    parser.add_argument(
        '--repo-url',
        dest='repo_url',
        help='Git repository URL (metadata override).'
    )

    parser.add_argument(
        '--repo-branch',
        dest='repo_branch',
        help='Git repository branch name (metadata override).'
    )

    parser.add_argument(
        '--date',
        dest='build_datetime',
        help='Datetime string describing when the document was authored '
             '(metadata override; the current datetime is used by default).'
    )

    parser.add_argument(
        '--extra-downloads',
        dest='extra_downloads',
        nargs='*',
        help='Paths of additional files to provide '
             'download links for from the landing page. '
             'These files will be copied into the root deployment directory. '
             'The main PDF (--pdf) should not be included in this list.'
    )

    parser.add_argument(
        '--docushare-url',
        dest='docushare_url',
        help='Docushare URL. Prefer the landing page URL in docushare rather '
             'than the direct PDF link so that users can see metadata about '
             'the accepted version. Using ls.st, this is the '
             'https://ls.st/ldm-nnn* URL.'
    )

    parser.add_argument(
        '--ltd-product',
        dest='ltd_product',
        help='Product name on LSST the Docs, such as `ldm-000` '
             '(metadata override).'
    )

    parser.add_argument(
        '--keeper-url',
        dest='keeper_url',
        help='LTD Keeper API URL (or $LTD_KEEPER_URL)'
    )

    parser.add_argument(
        '--keeper-user',
        dest='keeper_user',
        help='LTD Keeper API username (or $LTD_KEEPER_USER)'
    )

    parser.add_argument(
        '--keeper-password',
        dest='keeper_password',
        help='LTD Keeper API password (or $LTD_KEEPER_PASSWORD)'
    )

    parser.add_argument(
        '--aws-id',
        dest='aws_id',
        help='AWS key ID (or $LTD_AWS_ID)'
    )

    parser.add_argument(
        '--aws-secret',
        dest='aws_secret',
        help='AWS secret key (or $LTD_AWS_SECRET)'
    )

    parser.add_argument(
        '--upload',
        dest='upload',
        default=False,
        action='store_true',
        help='Upload built documentation to LSST the Docs'
    )

    return parser.parse_args()


def main():
    """Entrypoint for ``lander`` executable."""
    args = parse_args()
    config_logger(args)
    logger = structlog.get_logger(__name__)

    if args.show_version:
        # only print the version
        print_version()
        sys.exit(0)

    version = pkg_resources.get_distribution('lander').version
    logger.info('Lander version {0}'.format(version))

    # Disable any build from a fork that doesn't have access to secure configs
    if os.getenv('TRAVIS_SECURE_ENV_VARS') == 'true':
        logger.info('Skipping build from fork.')
        sys.exit(0)

    config = Configuration(args=args)

    # disable any build confirmed to be a PR with Travis
    if config['is_travis_pull_request']:
        logger.info('Skipping build from PR.')
        sys.exit(0)

    lander = Lander(config)
    lander.build_site()
    logger.info('Build complete')

    if config['upload']:
        lander.upload_site()
        logger.info('Upload complete')

    logger.info('Lander complete')


def config_logger(args):
    # Configure the root logger
    stream_handler = logging.StreamHandler()
    stream_formatter = logging.Formatter(
        '%(asctime)s %(levelname)8s %(name)s | %(message)s')
    stream_handler.setFormatter(stream_formatter)
    root_logger = logging.getLogger()
    root_logger.addHandler(stream_handler)
    root_logger.setLevel(logging.WARNING)

    # Setup first-party logging
    app_logger = logging.getLogger('lander')
    lsstprojectmeta_logger = logging.getLogger('lsstprojectmeta')
    ltdconveyor_logger = logging.getLogger('ltdconveyor')
    if args.verbose:
        app_logger.setLevel(logging.DEBUG)
        lsstprojectmeta_logger.setLevel(logging.DEBUG)
        ltdconveyor_logger.setLevel(logging.DEBUG)
    else:
        app_logger.setLevel(logging.INFO)
        lsstprojectmeta_logger.setLevel(logging.INFO)
        ltdconveyor_logger.setLevel(logging.INFO)

    # Configure structlog
    structlog.configure(
        processors=[
            structlog.stdlib.filter_by_level,
            structlog.stdlib.add_logger_name,
            structlog.stdlib.add_log_level,
            structlog.stdlib.PositionalArgumentsFormatter(),
            structlog.processors.StackInfoRenderer(),
            structlog.processors.format_exc_info,
            structlog.processors.UnicodeDecoder(),
            structlog.stdlib.render_to_log_kwargs,
        ],
        context_class=dict,
        logger_factory=structlog.stdlib.LoggerFactory(),
        wrapper_class=structlog.stdlib.BoundLogger,
        cache_logger_on_first_use=True,
    )


def print_version():
    version = pkg_resources.get_distribution('lander').version
    print(version)



"""Coordination infrastructure for making a landing page."""

import os
import shutil

import structlog
from lsstprojectmeta.jsonld import encode_jsonld

from .renderer import create_jinja_env, render_homepage
from . import ltdclient


class Lander(object):
    """Lander coordinates the creation and upload of a landing page for
    a PDF document.
    """

    def __init__(self, config):
        super().__init__()
        self._config = config
        self._logger = structlog.get_logger(__name__)
        self._jinja_env = create_jinja_env()

    def build_site(self):
        if not os.path.isdir(self._config['build_dir']):
            os.makedirs(self._config['build_dir'])

        # Define name of PDF file relative to index.html in built site
        relative_pdf_path = os.path.basename(self._config['pdf_path'])
        self._config['relative_pdf_path'] = relative_pdf_path

        # Add data for HTML title/description tags
        self._config['page_title'] = self._config['title_plain']
        self._config['page_description'] = self._config['abstract_plain']

        # Copy assets (css, js)
        # This algorithm is slightly naieve; we copy the whole built
        # assets directory everytime.
        asset_src_dir = os.path.join(os.path.dirname(__file__), 'assets')
        asset_dest_dir = os.path.join(self._config['build_dir'], 'assets')
        if os.path.isdir(asset_dest_dir):
            shutil.rmtree(asset_dest_dir)
        shutil.copytree(asset_src_dir, asset_dest_dir)

        # Copy PDF
        shutil.copy(self._config['pdf_path'],
                    os.path.join(self._config['build_dir'],
                                 relative_pdf_path))

        # Copy extra downloads. Also determine type for the template to
        # choose an appropriate icon.
        relative_extra_downloads = []
        for download_path in self._config['extra_downloads']:
            relative_path = os.path.basename(download_path)
            shutil.copy(download_path,
                        os.path.join(self._config['build_dir'],
                                     relative_path))
            # determine a type to choose the octicon
            ext = os.path.splitext(relative_path)[-1]
            if ext == ['.pdf']:
                download_type = 'pdf'
            elif ext in ['.tex', '.md', '.txt', '.rst']:
                download_type = 'text'
            elif ext in ['.gz', '.zip']:
                download_type = 'zip'
            elif ext in ['.tif', '.tiff', '.jpg', '.jpeg', '.png', '.gif']:
                download_type = 'media'
            elif ext in ['.py', '.h', '.c', '.cpp', '.ipynb', '.json']:
                download_type = 'code'
            else:
                download_type = 'file'
            relative_extra_downloads.append({
                'path': relative_path,
                'type': download_type
            })
        self._config['relative_extra_downloads'] = relative_extra_downloads

        # Write index.html page
        index_html = render_homepage(self._config, self._jinja_env)
        index_html_path = os.path.join(self._config['build_dir'], 'index.html')
        with open(index_html_path, mode='w', encoding='utf-8') as f:
            f.write(index_html)

        # Write metadata file
        jsonld_path = os.path.join(self._config['build_dir'],
                                   'metadata.jsonld')
        self.write_metadata(jsonld_path)

    def write_metadata(self, output_path):
        """Build a JSON-LD dataset for LSST Projectmeta.

        Parameters
        ----------
        output_path : `str`
            File path where the ``metadata.jsonld`` should be written for the
            build.
        """
        if self._config.lsstdoc is None:
            self._logger.info('No known LSST LaTeX source (--tex argument). '
                              'Not writing a metadata.jsonld file.')
            return

        # Build a JSON-LD dataset for the report+source repository.
        product_data = ltdclient.get_product(self._config)
        metadata = self._config.lsstdoc.build_jsonld(
            url=product_data['published_url'],
            code_url=product_data['doc_repo'],
            ci_url='https://travis-ci.org/' + self._config['github_slug'],
            readme_url=None,
            license_id=None)

        json_text = encode_jsonld(
            metadata,
            separators=(',', ':'),  # compact
            ensure_ascii=False)  # unicode output
        with open(output_path, 'w') as f:
            f.write(json_text)

    def upload_site(self):
        """Upload a previously-built site to LSST the Docs."""
        if not os.path.isdir(self._config['build_dir']):
            message = 'Site not built at {0}'.format(self._config['build_dir'])
            self._logger.error(message)
            raise RuntimeError(message)

        ltdclient.upload(self._config)



"""Configuration facilities.
"""
import json
import sys
import os
from collections import ChainMap
import re
import datetime
import urllib.parse

from lsstprojectmeta.tex.lsstdoc import LsstLatexDoc
import structlog
import dateutil
import requests

from .exceptions import DocuShareError


# Detects a GitHub repo slug from a GitHub URL
GITHUB_SLUG_PATTERN = re.compile(
    r"https://github.com"
    r"/(?P<org>[a-z0-9\-_~%!$&'()*+,;=:@]+)"
    r"/(?P<name>[a-z0-9\-_~%!$&'()*+,;=:@]+)"
)


# Regular expression that matches docushare release tags
# E.g. docushare-v1
DOCUSHARE_TAG_PATTERN = re.compile(
    r"^docushare-v(?P<number>\d+$)"
)


class Configuration(object):
    """lander configuration container and validator.

    Parameters
    ----------
    args : `argparse.Namespace`
        An argument parser Namespace. This is made by the
        `argparse.ArgumentParser.parse_args` method.
    _validate_pdf : `bool`, optional
        Validate that the PDF exists. Default is `True`.
    **config : keyword arguments
        Additional keyword arguments that override configurations from the
        ``args``.
    """

    def __init__(self, args=None, _validate_pdf=True, **config):
        super().__init__()
        self._logger = structlog.get_logger(__name__)

        # Make dict from argparse namespace
        if args is not None:
            self._args = {k: v for k, v in vars(args).items() if v}
        else:
            self._args = {}

        # Holds configuration overrides and computed configurations
        self._configs = dict(config)

        # Default configurations
        self._defaults = self._init_defaults()

        # Configurations from environment variables
        self._envvars = self._get_environment_variables()

        # Configurations are merged in a ChainMap. Priority is
        # 1. Computed configurations.
        # 2. Command line variables.
        # 3. Environment variables.
        # 4. Defaults.
        self._chain = ChainMap(self._configs, self._args,
                               self._envvars, self._defaults)

        # Validate inputs
        if _validate_pdf:
            self._validate_pdf_file()

        # Get metadata from the TeX source
        self.lsstdoc = None
        if self['lsstdoc_tex_path'] is not None:
            if not os.path.exists(self['lsstdoc_tex_path']):
                self._logger.error('Cannot find {0}'.format(
                    self['lsstdoc_tex_path']))
                sys.exit(1)

            # Extract metadata from the LsstLatexDoc document
            self.lsstdoc = LsstLatexDoc.read(self['lsstdoc_tex_path'])
            self['title'] = self.lsstdoc.title
            self['title_html'] = self.lsstdoc.html_title
            self['title_plain'] = self.lsstdoc.plain_title
            self['build_datetime'] = self.lsstdoc.revision_datetime
            if self.lsstdoc.abstract is not None:
                self['abstract'] = self.lsstdoc.abstract
                self['abstract_html'] = self.lsstdoc.html_abstract
                self['abstract_plain'] = self.lsstdoc.plain_abstract
            if self.lsstdoc.handle is not None:
                self['doc_handle'] = self.lsstdoc.handle
                self['series'] = self.lsstdoc.series
                self['series_name'] = self._get_series_name(self['series'])
            if self.lsstdoc.authors is not None:
                self['authors'] = self.lsstdoc.authors
                self['authors_html'] = self.lsstdoc.html_authors
                self['authors_plain'] = self.lsstdoc.plain_authors

        # Get metadata from Travis environment
        if self['environment'] is not None:
            self['git_commit'] = os.getenv('TRAVIS_COMMIT')
            self['git_branch'] = os.getenv('TRAVIS_BRANCH')
            self['git_tag'] = os.getenv('TRAVIS_TAG')
            self['github_slug'] = os.getenv('TRAVIS_REPO_SLUG')
            self['travis_job_number'] = os.getenv('TRAVIS_JOB_NUMBER')
            self['travis_build_web_url'] = os.getenv('TRAVIS_BUILD_WEB_URL')
            if os.getenv('TRAVIS_PULL_REQUEST').lower() == 'false':
                self['is_travis_pull_request'] = False
            else:
                self['is_travis_pull_request'] = True

        # Apply metadata overrides

        if 'title' in self._args:
            self['title'] = self._args['title']

        if 'authors_json' in self._args:
            author_list = json.loads(self._args['authors_json'])
            self['authors'] = author_list

        if 'doc_handle' in self._args:
            self['doc_handle'] = self._args['doc_handle']
            self['series'] = self['doc_handle'].split('-', 1)[0]
            self['series_name'] = self._get_series_name(self['series'])

        if 'abstract' in self._args:
            self['abstract'] = self._args['abstract']

        if 'repo_url' in self._args:
            self['repo_url'] = self._args['repo_url']

            # extract github repo slug from repo_url
            match = GITHUB_SLUG_PATTERN.match(self['repo_url'])
            if match:
                self['github_slug'] = '/'.join((
                    match.group('org'),
                    match.group('name')))

        if 'extra_downloads' in self._args:
            self['extra_downloads'] = self._args['extra_downloads']

        if 'git_branch' in self._args:
            self['git_branch'] = self._args['git_branch']

        if 'build_datetime' in self._args:
            parsed_datetime = dateutil.parser.parse(
                self._args['build_datetime'])
            if parsed_datetime.tzinfo is None:
                parsed_datetime = parsed_datetime.replace(
                    tzinfo=dateutil.tz.tzutc())
            self['build_datetime'] = parsed_datetime

        # Get the DocuShare URL if not set already
        if self['doc_handle'] is not None and self['docushare_url'] is None:
            try:
                self['docushare_url'] = self._get_docushare_url(
                    self['doc_handle'], validate=True)
            except DocuShareError as e:
                message = 'Could not compute DocuShare URL for {0}'.format(
                    self['doc_handle'])
                self._logger.warning(message)
                self._logger.warning(str(e))

        self['is_draft_branch'] = self._determine_draft_status(
            self['git_branch'],
            lsstdoc=self.lsstdoc)

        # Post configuration validation
        if self['upload']:
            if self['ltd_product'] is None:
                message = '--ltd-product must be set for uploads'
                self._logger.error(message)
                sys.exit(1)

            if self['aws_id'] is None:
                message = '--aws-id must be set for uploads'
                self._logger.error(message)
                sys.exit(1)

            if self['aws_secret'] is None:
                message = '--aws-secret must be set for uploads'
                self._logger.error(message)
                sys.exit(1)

            if self['keeper_url'] is None:
                message = '--keeper-url must be set for uploads'
                self._logger.error(message)
                sys.exit(1)

            if self['keeper_user'] is None:
                message = '--keeper-user must be set for uploads'
                self._logger.error(message)
                sys.exit(1)

            if self['keeper_password'] is None:
                message = '--keeper-password must be set for uploads'
                self._logger.error(message)
                sys.exit(1)

    def __getitem__(self, key):
        """Access configurations, first from the explicitly set configurations,
        and secondarily from the command line arguments.
        """
        return self._chain[key]

    def __setitem__(self, key, value):
        self._chain[key] = value

    def _validate_pdf_file(self):
        """Validate that the pdf_path configuration is set and the referenced
        file exists.

        Exits the program with status 1 if validation fails.
        """
        if self['pdf_path'] is None:
            self._logger.error('--pdf argument must be set')
            sys.exit(1)
        if not os.path.exists(self['pdf_path']):
            self._logger.error('Cannot find PDF ' + self['pdf_path'])
            sys.exit(1)

    def _get_series_name(self, series):
        series_names = {
            'sqr': 'SQuaRE Technical Note',
            'dmtn': 'Data Management Technical Note',
            'smtn': 'Simulations Technical Note',
            'ldm': 'LSST Data Management',
            'lse': 'LSST Systems Engineering',
            'lpm': 'LSST Project Management',
        }
        return series_names.get(series.lower(), '')

    @staticmethod
    def _get_docushare_url(handle, validate=True):
        """Get a docushare URL given document's handle.

        Parameters
        ----------
        handle : `str`
            Handle name, such as ``'LDM-151'``.
        validate : `bool`, optional
            Set to `True` to request that the link resolves by performing
            a HEAD request over the network. `False` disables this testing.
            Default is `True`.

        Returns
        -------
        docushare_url : `str`
            Shortened DocuShare URL for the document corresponding to the
            handle.

        Raises
        ------
        lander.exceptions.DocuShareError
            Raised for any error related to validating the DocuShare URL.
        """
        logger = structlog.get_logger(__name__)
        logger.debug('Using Configuration._get_docushare_url')

        # Make a short link to the DocuShare version page since
        # a) It doesn't immediately trigger a PDF download,
        # b) It gives the user extra information about the document before
        #    downloading it.
        url = 'https://ls.st/{handle}*'.format(handle=handle.lower())

        if validate:
            # Test that the short link successfully resolves to DocuShare
            logger.debug('Validating {0}'.format(url))
            try:
                response = requests.head(url, allow_redirects=True, timeout=30)
            except requests.exceptions.RequestException as e:
                raise DocuShareError(str(e))

            error_message = 'URL {0} does not resolve to DocuShare'.format(url)
            if response.status_code != 200:
                logger.warning('HEAD {0} status: {1:d}'.format(
                    url, response.status_code))
                raise DocuShareError(error_message)
            redirect_url_parts = urllib.parse.urlsplit(response.url)
            if redirect_url_parts.netloc != 'docushare.lsst.org':
                logger.warning('{0} resolved to {1}'.format(url, response.url))
                raise DocuShareError(error_message)

        return url

    @staticmethod
    def _determine_draft_status(git_branch, lsstdoc=None):
        """Determine if the document build is considered a draft based on
        DM's policies given the Git branch or draft status in the lsstdoc-class
        LaTeX document.

        Parameters
        ----------
        git_branch : `str`
            Git branch or tag name.
        lsstdoc : `lsstprojectmeta.tex.lsstdoc.LsstLatexDoc`
            `~lsstprojectmeta.tex.lsstdoc.LsstLatexDoc` instance where, if the
            ``is_draft`` argument is `True`, the draft status is True.

        Returns
        -------
        is_draft : `bool`
            Returns `True` if the build is a draft, and `False` if it is
            not a draft.

        Notes
        -----
        The document **is not** considered a draft if:

        - Branch is ``master``
        - If ``lsstdoc`` is provided and ``lsstdoc.is_draft == False``,
          meaning that the ``lsstdoc`` option is not included in the document's
          class options.
        """
        if git_branch == 'master':
            return False

        if lsstdoc is not None:
            return lsstdoc.is_draft

        return True

    def _init_defaults(self):
        """Create a `dict` of default configurations."""
        defaults = {
            'build_dir': None,
            'build_datetime': datetime.datetime.now(dateutil.tz.tzutc()),
            'pdf_path': None,
            'extra_downloads': list(),
            'environment': None,
            'lsstdoc_tex_path': None,
            'title': None,
            'title_plain': "",
            'authors': None,
            'authors_json': list(),
            'doc_handle': None,
            'series': None,
            'series_name': None,
            'abstract': None,
            'abstract_plain': "",
            'ltd_product': None,
            'docushare_url': None,
            'github_slug': None,
            'git_branch': 'master',  # so we default to the main LTD edition
            'git_commit': None,
            'git_tag': None,
            'travis_job_number': None,
            'is_travis_pull_request': False,  # If not on Travis, not a PR
            'is_draft_branch': True,
            'aws_id': None,
            'aws_secret': None,
            'keeper_url': 'https://keeper.lsst.codes',
            'keeper_user': None,
            'keeper_password': None,
            'upload': False
        }
        return defaults

    def _get_environment_variables(self):
        var_keys = {
            'aws_id': 'LTD_AWS_ID',
            'aws_secret': 'LTD_AWS_SECRET',
            'keeper_url': 'LTD_KEEPER_URL',
            'keeper_user': 'LTD_KEEPER_USER',
            'keeper_password': 'LTD_KEEPER_PASSWORD',
        }
        env_configs = {}
        for config_key, var_name in var_keys.items():
            env_value = os.getenv(var_name)
            if env_value is not None:
                env_configs[config_key] = env_value
        return env_configs



"""HTML landing page generator for LSST PDF documentation deployed from Git to
LSST the Docs.

https://github.com/lsst-sqre/lander
"""
from pkg_resources import get_distribution, DistributionNotFound
try:
    __version__ = get_distribution('lander').version
except DistributionNotFound:
    # package is not installed
    pass



"""Client for uploading to LSST the Docs.
"""

import requests
import ltdconveyor


def upload(config):
    """Upload the build documentation site to LSST the Docs.

    Parameters
    ----------
    config : `lander.config.Configuration`
        Site configuration, which includes upload information and credentials.
    """
    token = get_keeper_token(config['keeper_url'],
                             config['keeper_user'],
                             config['keeper_password'])
    build_resource = register_build(config, token)

    ltdconveyor.upload_dir(
        build_resource['bucket_name'],
        build_resource['bucket_root_dir'],
        config['build_dir'],
        aws_access_key_id=config['aws_id'],
        aws_secret_access_key=config['aws_secret'],
        surrogate_key=build_resource['surrogate_key'],
        cache_control='max-age=31536000',
        surrogate_control=None,
        upload_dir_redirect_objects=True)

    confirm_build(config, token, build_resource)


def get_keeper_token(base_url, username, password):
    """Get a temporary auth token from LTD Keeper."""
    token_endpoint = base_url + '/token'
    r = requests.get(token_endpoint, auth=(username, password))
    if r.status_code != 200:
        raise RuntimeError('Could not authenticate to {0}: error {1:d}\n{2}'.
                           format(base_url, r.status_code, r.json()))
    return r.json()['token']


def register_build(config, keeper_token):
    data = {
        'git_refs': [config['git_branch']],
    }

    r = requests.post(
        config['keeper_url'] + '/products/{p}/builds/'.format(
            p=config['ltd_product']),
        auth=(keeper_token, ''),
        json=data)

    if r.status_code != 201:
        raise RuntimeError(r.json())
    build_info = r.json()
    return build_info


def confirm_build(config, keeper_token, build_resource):
    build_url = build_resource['self_url']
    r = requests.patch(build_url,
                       auth=(keeper_token, ''),
                       json={'uploaded': True})
    if r.status_code != 200:
        raise RuntimeError(r)


def get_product(config):
    """Get the /product/<product> resource from LTD Keeper.
    """
    product_url = config['keeper_url'] + '/products/{p}'.format(
        p=config['ltd_product'])
    r = requests.get(product_url)
    if r.status_code != 200:
        raise RuntimeError(r.json())
    product_info = r.json()
    return product_info



from pathlib import Path

from setuptools import setup


packagename = 'lander'
description = ('HTML landing page generator for LSST PDF documentation '
               'deployed from Git to LSST the Docs.')
author = 'Association of Universities for Research in Astronomy, Inc.'
author_email = 'jsick@lsst.org'
license = 'MIT'
url = 'https://github.com/lsst-sqre/lander'
long_description = (Path(__file__).parent / 'README.rst').read_text()


setup(
    name=packagename,
    description=description,
    long_description=long_description,
    url=url,
    author=author,
    author_email=author_email,
    license=license,
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    keywords='lsst',
    packages=['lander'],
    use_scm_version=True,
    setup_requires=['setuptools_scm'],
    install_requires=[
        'python-dateutil>=2.6.0',
        'Jinja2==2.9.6',
        'structlog==17.1.0',
        'ltd-conveyor==0.3.1',
        'requests==2.14.2',
        'lsst-projectmeta-kit==0.3.3'
    ],
    extras_require={
        'dev': [
            # Development/testing dependencies
            'pytest==4.2.1',
            'pytest-cov==2.6.1',
            'pytest-flake8==1.0.4'
        ]},
    package_data={'lander': [
        'assets/*.svg',
        'assets/*.css',
        'assets/*.js',
        'templates/*'
    ]},
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'lander = lander.main:main'
        ]
    }
)



"""HTML renderering functions."""

import os
import re

import jinja2


def create_jinja_env():
    """Create a Jinja2 `~jinja2.Environment`.

    Returns
    -------
    env : `jinja2.Environment`
        Jinja2 template rendering environment, configured to use templates in
        ``templates/``.
    """
    template_dir = os.path.join(os.path.dirname(__file__), 'templates')
    env = jinja2.Environment(
        loader=jinja2.FileSystemLoader(template_dir),
        autoescape=jinja2.select_autoescape(['html'])
    )
    env.filters['simple_date'] = filter_simple_date
    env.filters['paragraphify'] = filter_paragraphify
    return env


def render_homepage(config, env):
    """Render the homepage.jinja template."""
    template = env.get_template('homepage.jinja')
    rendered_page = template.render(
        config=config)
    return rendered_page


def filter_simple_date(value):
    """Filter a `datetime.datetime` into a 'YYYY-MM-DD' string."""
    return value.strftime('%Y-%m-%d')


def filter_paragraphify(value):
    """Convert text into one or more paragraphs, including <p> tags.

    Based on https://gist.github.com/cemk/1324543
    """
    value = re.sub(r'\r\n|\r|\n', '\n', value)  # Normalize newlines
    paras = re.split('\n{2,}', value)
    paras = ['<p>{0}</p>'.format(p) for p in paras if len(p) > 0]
    return jinja2.Markup('\n\n'.join(paras))
